-------------------------------------------------------------------------------
--                L  O  C  A  L     V  A  R  I  A  B  L  E  S
-------------------------------------------------------------------------------

local addonName, L = ...;

--
-- Function to return the index for any undefined indexes
--
local function defaultValueIndex(table, index)
  return index;
end

-- Set the function for any undefined indexes
setmetatable(L, {__index = defaultValueIndex});

local NUM_INVITES_PER_PAGE = 10;

local daysInYear =
{
  {0, 31, 60, 91, 121, 152, 182, 213, 244, 274, 305, 335},
  {0, 31, 59, 90, 120, 151, 181, 212, 243, 273, 304, 334}
};

-------------------------------------------------------------------------------
--                L  O  C  A  L     F  U  N  C  T  I  O  N  S
-------------------------------------------------------------------------------

--
-- Function to disable a Niggles character list frame and its descendants
--
local function characterListDisable(self)
  self:SetEnabled(false);
  return;
end

--
-- Function to enable a Niggles character list frame and its descendants
--
local function characterListEnable(self)
  self:SetEnabled(true);
  return;
end

--
-- Function to set the enabled state of a Niggles character list frame and
-- its descendants
--
local function characterListSetEnabled(self, enable)
  -- Local Variables
  local buttons;

  -- Save the enabled status
  self.isEnabled = enable;

  -- Enable/Disable the label
  self.label:SetFontObject(enable and self.label.normalFont or
    self.label.disableFont);

  -- Enable/Disable the scroll frame and its buttons
  self.scrollFrame.ScrollBar:SetEnabled(enable);
  buttons = self.scrollFrame.buttons;
  if (buttons ~= nil) then
    for idx = 1, #buttons do
      buttons[idx]:SetEnabled(enable);
    end
  end
  self.scrollFrame.update();

  -- Enable/Disable the edit box and Add button
  self.edit:SetEnabled(enable);
  self.add:SetEnabled(enable);

  return;
end

-------------------------------------------------------------------------------
--               G  L  O  B  A  L     V  A  R  I  A  B  L  E  S
-------------------------------------------------------------------------------

NigglesHolidayBrewfest        = 0x00000001;
NigglesHolidayChildrensWeek   = 0x00000002;
NigglesHolidayDarkmoonFaire   = 0x00000004;
NigglesHolidayDayOfTheDead    = 0x00000008;
NigglesHolidayHallowsEnd      = 0x00000010;
NigglesHolidayHarvestFestival = 0x00000020;
NigglesHolidayLoveInTheAir    = 0x00000040;
NigglesHolidayLunarFestival   = 0x00000080;
NigglesHolidayMidsummer       = 0x00000100;
NigglesHolidayNobleGarden     = 0x00000200;
NigglesHolidayPiratesDay      = 0x00000400;
NigglesHolidayWinterVeil      = 0x00000800;

-------------------------------------------------------------------------------
--               G  L  O  B  A  L     F  U  N  C  T  I  O  N  S
-------------------------------------------------------------------------------

--
-- Function to initialise a Niggles Character List
--
function NigglesCharacterListInit(frame, label, buttonTemplate, updateFunc, addFunc)
  -- Save some of the arguments
  frame.buttonTemplate = buttonTemplate;
  frame.addFunc        = addFunc;

  -- Set the frame's label
  frame.label:SetText(label);
  frame.label.normalFont  = "GameFontNormal";
  frame.label.disableFont = "GameFontDisable";

  -- Initialise the scroll frame
  frame.scrollFrame.update = updateFunc;
  frame.scrollFrame.scrollBar.doNotHide = true;

  -- Adjust some colours
  frame.scrollFrame.ScrollBar.trackBG:SetVertexColor(0.2, 0.13, 0.08, 0.25);
  frame.border:SetBackdropBorderColor(0.75, 0.75, 0.75, 1.0);

  return;
end

--
-- Function to process the Enter key in an Character List edit box
--
function NigglesCharacterListEditOnEnterPressed(self)
  -- Local Variables
  local name;
  local frame = self:GetParent();

  -- Check the AutoComplete frame doesn't want the key press
  if (not AutoCompleteEditBox_OnEnterPressed(self)) then
    -- Get the name entered by the user
    name = strtrim(self:GetText());
    if (name ~= "") then
      -- Capitalize the name
      name = (name:sub(1,1):upper())..(name:sub(2):lower());

      -- Call the add function
      if (frame.addFunc ~= nil) then
        frame.addFunc(frame, name);
      end

      -- Clear the edit frame
      self:SetText("");
      AutoComplete_HideIfAttachedTo(self)
    else
      self:ClearFocus();
    end
  end

  return;
end

--
-- Function to tidy up when a Character List Edit frame loses focus
--
function NigglesCharacterListEditOnFocusLost(self)
  AutoCompleteEditBox_OnEditFocusLost(self);
  self:HighlightText(0, 0);
  local trimmedText = strtrim(self:GetText());
  if (trimmedText == "") then
    self:SetText("");
  end

  return;
end

--
-- Function called when a Niggles Character List scroll frame changes size
--
-- NOTE: The buttons are created when the frame changes size so the character
--       list doesn't have to be fixed
--
function NigglesCharacterListScrollFrameOnSize(self)
  -- Local Variables
  local parent = self:GetParent();
  local isEnabled;
  local buttons;

  -- Check a button template has been supplied
  if (parent.buttonTemplate ~= nil) then
    -- Create buttons, if required
    HybridScrollFrame_CreateButtons(self,
      self:GetParent().buttonTemplate, 1, 0);

    -- Enable/Disable the buttons
    for _, button in ipairs(self.buttons) do
      button:SetEnabled(parent.isEnabled);
    end

    -- Update the scroll frame, if possible
    if (self.update ~= nil) then
      self.update();
    end
  end

  return;
end

--
-- Function called when a Niggles Character List is created
--
function NigglesCharacterListOnLoad(self)
  -- Add some extra methods to the frame
  self.Disable    = characterListDisable;
  self.Enable     = characterListEnable;
  self.SetEnabled = characterListSetEnabled;

  return;
end

--
-- Function to enable/disable a frame and all its descendants. This function
-- is recursive.
--
function NigglesEnableFrame(frame, enable)
  -- Local Variables
  local childType;
  local frameType;

  -- Enable/disable the frame, if possible
  if ((frame.Enable) and (enable == true)) then
    frame:Enable();
  elseif ((frame.Disable) and (enable == false)) then
    frame:Disable();
  elseif (frame.GetObjectType) then
    frameType = frame:GetObjectType();
    if (frameType == "Frame") then
      -- Process each region
      for _, child in ipairs({frame:GetRegions()}) do
        -- Get the child's type
        childType = child:GetObjectType();

        -- Process the child, based on its type
        if (childType == "FontString") then
          if ((child.normalFont ~= nil) and (enable == true)) then
            child:SetFontObject(child.normalFont);
          elseif ((child.disableFont ~= nil) and (enable == false)) then
            child:SetFontObject(child.disableFont);
          end
        end
      end
    elseif (frameType == "EditBox") then
      frame:EnableJoystick(enable);
      frame:EnableMouse(enable);
      frame:EnableMouseWheel(enable);
      frame:EnableKeyboard(enable);
      frame:ClearFocus();
      if ((frame.normalFont ~= nil) and (enable == true)) then
        frame:SetFontObject(frame.normalFont);
      elseif ((frame.disableFont ~= nil) and (enable == false)) then
        frame:SetFontObject(frame.disableFont);
      end
    end
  end

  -- Process all children of the current frame
  for _, child in ipairs({frame:GetChildren()}) do
    NigglesEnableFrame(child, enable);
  end

  return;
end

--
-- Function to get the localized version of a string
--
function NigglesGetLocaleString(string)
  return L[string];
end

--
-- Function to (roughly) convert a date & time into a number of minutes
--
function NigglesMinutes(year, month, day, hour, min)
  -- If no date/time has been specified...
  if (year == nil) then
    -- ...use the current date/time
    _, month, day, year = CalendarGetDate();
    hour, min = GetGameTime();
  end

  -- Work out the number of minutes
  return (((((year*365)+math.floor((year-1)/4)+
    daysInYear[((year-1)%4 == 0 and 2 or 1)][month]+day)*24)+hour)*60)+min;
end

--
-- Function to get all the points set for a frame and store them in an array
--
function NigglesFramePointsGet(frame, array)
  -- Clear the array
  wipe(array);

  -- Get all the frame's points
  for pointIdx = 1, frame:GetNumPoints() do
    array[pointIdx] = {frame:GetPoint(pointIdx)};
  end

  return;
end

--
-- Function to set all the points specified in an array for a frame
--
function NigglesFramePointsSet(frame, array)
  -- Clear all the frame's points
  frame:ClearAllPoints();

  -- Set all the frame's points
  for pointIdx = 1, #array do
    frame:SetPoint(array[pointIdx][1], array[pointIdx][2], array[pointIdx][3],
      array[pointIdx][4], array[pointIdx][5]);
  end

  return;
end

--
-- Function to check if a value is in a table, ignoring case
--
function NigglesTableContains(table, value)
  -- Local Variables
  local idx = 1;

  -- Search the table for the specified value
  value = value:lower();
  while (table[idx]) do
    if (value == table[idx]:lower()) then
      return 1;
    end
    idx = idx+1;
  end

  return nil;
end

--
-- Function to set the contents of the Niggles tooltip. It clears any old lines
-- and then sets the new contents. It then checks if new lines have been added
-- to the tooltip. If so then the lines are added to the arrays used to scan
-- the tooltip's contents more efficiently.
--
-- NOTE: While the tooltip's normal 'Set...' methods could be hooked, there
--       are so many that it was deemed better to use this technique to keep
--       the arrays of text lines up to date.
--
function NigglesTooltipSetContents(self, funcName, ...)
  -- Local Variables
  local numLines;
  local numElements = #(self.textLeft);

  -- Clear any old content
  self:ClearLines();

  -- Set the tooltip's new contents
  (self[funcName])(self, ...);

  -- Check if the tooltip has new lines
  numLines = self:NumLines();
  if (numLines > numElements) then
    -- Add the new lines to the arrays used to scan the tooltip
    for lineIdx = numElements+1, numLines do
      self.textLeft[lineIdx]  = _G["NigglesTooltipTextLeft"..lineIdx];
      self.textRight[lineIdx] = _G["NigglesTooltipTextRight"..lineIdx];
    end
  end

  return numLines;
end

--
-- Function called when the Niggles tooltip is loaded. It sets the owner for
-- the tooltip, initialise the arrays used to scan the tooltip and adds a
-- method for setting the contents of the tooltip.
--
function NigglesTooltipOnLoad(self)
  -- Set the tooltip's owner
  self:SetOwner(WorldFrame, "ANCHOR_NONE");

  -- Initialise the arrays used to scan the tooltip's contents
  self.textLeft  = {_G["NigglesTooltipTextLeft1"]};
  self.textRight = {_G["NigglesTooltipTextRight1"]};

  -- Add a new method for setting the contents of the tooltip
  self.SetContents = NigglesTooltipSetContents;

  return;
end

--
-- Function to initialise the Niggles Warning popup
--
function NigglesWarningPopupInit(text, postscript, okLabel, cancelLabel, callback)
  -- Local Variables
  local popup;

  -- Create the Niggles warning popup, if required
  if (NigglesWarningPopup == nil) then
    CreateFrame("Frame", "NigglesWarningPopup", UIParent,
      "NigglesWarningPopupTemplate");
  end
  popup = NigglesWarningPopup;

  -- Set the warning popup's text
  popup.text:SetText(text);
  popup.postscript:SetShown((postscript ~= nil));
  if (postscript ~= nil) then
    popup.postscript:SetText(tostring(postscript));
  end

  -- Set the buttons up
  popup.buttons.ok:SetText(tostring(okLabel));
  popup.buttons.cancel:SetShown((cancelLabel ~= nil));
  if (cancelLabel ~= nil) then
    popup.buttons.cancel:SetText(tostring(cancelLabel));
    popup.buttons:SetWidth((popup.buttons.cancel:GetWidth()*2)+10);
  else
    popup.buttons:SetWidth(popup.buttons.ok:GetWidth());
  end
  if (type(callback) == "function") then
    popup.buttons.callback = callback;
  else
    popup.buttons.callback = StaticPopupSpecial_Hide;
  end

  -- Hide the optional parts of the popup
  popup.list:Hide();
  popup.item:Hide();

  return;
end

-------------------------------------------------------------------------------
--                 G  L  O  B  A  L     C  L  A  S  S  E  S
-------------------------------------------------------------------------------

--
-- Class to provide a history for values, such as achievements and recipes
--
NigglesHistoryClass =
{
  -- Class Variables
  maxElements = 16,

  -- Method to add an item to the history list
  Add = function(self, value)
    -- Check a value was specified
    if ((value ~= nil) and (value ~= "")) then
      -- Check if the list is empty
      if (self.base == 0) then
        self.base        = 1;
        self.current     = 1;
        self.top         = 1;
        self.elements[1] = value;
      elseif (self.elements[self.current] ~= value) then
        -- Update the current, top and base indices
        self.current = (self.current == self.maxElements and 1 or
          self.current+1);
        self.top = self.current;
        if (self.current == self.base) then
          self.base = (self.base == self.maxElements and 1 or self.base+1);
        end

        -- Add the value to the history
        self.elements[self.current] = value;
      end
    end

    return;
  end;

  --
  -- Method to clear the history
  --
  Clear = function(self)
    -- Set the base index to a special value to indicate the history is empty
    self.base = 0;
  end;

  --
  --
  -- Method to get the current value from the history list
  Get = function(self)
    return (self.base ~= 0 and self.elements[self.current] or nil);
  end;

  --
  -- Method to check if moving backward or forward is valid in the history
  --
  IsMoveValid = function(self, backward)
    -- Check if moving in the specified direction is valid
    if (backward) then
      return ((self.base ~= 0) and (self.current ~= self.base));
    else
      return ((self.base ~= 0) and (self.current ~= self.top));
    end
  end;

  --
  -- Method to move one element back or forward in the history list
  --
  Move = function(self, backward)
    -- Local Variables
    local value = nil;

    -- Check the history list isn't empty
    if (self.base ~= 0) then
      -- Move in the specified direction in the history list, if possible
      if ((backward) and (self.current ~= self.base)) then
        -- Decrement the current index
        self.current = (self.current == 1 and self.maxElements or
          self.current-1);
        value = self.elements[self.current];
      elseif ((not backward) and (self.current ~= self.top)) then
        -- Increment the current index
        self.current = (self.current == self.maxElements and
          1 or self.current+1);
        value = self.elements[self.current];
      end
    end

    -- Return the current recipe name
    return value;
  end;

  --
  -- Method to create a new class object
  --
  New = function(self, maxElements)
    -- Create a new table for the object
    local object =
    {
      base     = 0,
      top      = 0,
      current  = 0,
      elements = {},
    };

    -- Set the object's metatable to use the default class values
    setmetatable(object, self);
    self.__index = self;

    -- Save the maximum number of elements for the history, if specified
    if ((maxElements ~= nil) and
        (type(maxElements) == "number") and
        (maxElements > 1)) then
      object.maxElements = maxElements;
    end

    -- Return the new object
    return object;
  end;
};
