-------------------------------------------------------------------------------
--                L  O  C  A  L     V  A  R  I  A  B  L  E  S
-------------------------------------------------------------------------------

local addonName, L = ...;

local merchantSearch;
local merchantItemMasks = {};

-------------------------------------------------------------------------------
--                L  O  C  A  L     F  U  N  C  T  I  O  N  S
-------------------------------------------------------------------------------

--
-- Function used to update the 'Buyback' tab in the Merchant panel, based on
-- the search text
--
local function buybackTabUpdate(searchText)
  -- Local Variables
  local buttonIdx;
  local isKnown;
  local itemIdx;
  local itemMatch;
  local numItems = GetNumBuybackItems();
  local textLeft = NigglesTooltip.textLeft;

  -- Show/hide each button's mask, depending on the buttons' text
  for buttonIdx = 1, BUYBACK_ITEMS_PER_PAGE do
    -- Initialise some variables
    isKnown   = false;
    itemIdx   = buttonIdx;
    itemMatch = true;

    -- Check if the button is displaying an item
    if (itemIdx <= numItems) then
      -- Check if the button's item is already known
      if (NigglesDBC.settings.UiMerchantItemKnown) then
        numLines = NigglesTooltip:SetContents("SetBuybackItem", itemIdx);
        for lineIdx = numLines, math.max(numLines-2, 1), -1 do
          if (textLeft[lineIdx]:GetText() == ITEM_SPELL_KNOWN) then
            isKnown = true;
            break;
          end
        end
      end

      -- Check if the name of the button's item matches the search string
      if ((NigglesDBC.settings.UiMerchantSearch) and (searchText ~= "")) then
        if (string.find(string.lower(GetBuybackItemInfo(itemIdx) or ""),
          searchText, 1, true) == nil) then
          itemMatch = false;
        end
      end
    end

    -- Show/Hide textures for the button
    if ((isKnown) or (not itemMatch)) then
      merchantItemMasks[buttonIdx].isKnown:SetShown(isKnown);
      merchantItemMasks[buttonIdx].noMatchIcon:SetShown(not itemMatch);
      merchantItemMasks[buttonIdx].noMatchName:SetShown(not itemMatch);
      merchantItemMasks[buttonIdx]:Show();
    else
      merchantItemMasks[buttonIdx]:Hide();
    end
  end

  return;
end

--
-- Function used to update the 'Buyback' tab in the Merchant panel, based on
-- the search text
--
local function merchantTabUpdate(searchText)
  -- Local Variables
  local buttonIdx;
  local currencyLink;
  local isKnown;
  local itemIdx;
  local itemMatch;
  local itemOffset = ((MerchantFrame.page-1)*MERCHANT_ITEMS_PER_PAGE);
  local numCurrencies;
  local numItems = GetMerchantNumItems();
  local numLines;
  local textLeft = NigglesTooltip.textLeft;

  -- Show/hide each button's mask, depending on the buttons' text
  for buttonIdx = 1, MERCHANT_ITEMS_PER_PAGE do
    -- Initialise some variables
    isKnown   = false;
    itemIdx   = itemOffset+buttonIdx;
    itemMatch = true;

    -- Check if the button is displaying an item
    if (itemIdx <= numItems) then
      -- Check if the button's item is already known
      if (NigglesDBC.settings.UiMerchantItemKnown) then
        numLines = NigglesTooltip:SetContents("SetMerchantItem", itemIdx);
        for lineIdx = numLines, math.max(numLines-2, 1), -1 do
          if (textLeft[lineIdx]:GetText() == ITEM_SPELL_KNOWN) then
            isKnown = true;
            break;
          end
        end
      end

      -- Check if the name of the button's item matches the search string
      if ((NigglesDBC.settings.UiMerchantSearch) and (searchText ~= "")) then
        if (string.find(string.lower(GetMerchantItemInfo(itemIdx) or ""),
          searchText, 1, true) == nil) then
          itemMatch = false;
        end

        -- If the item's name doesn't match
        numCurrencies = GetMerchantItemCostInfo(itemIdx);
        if ((not itemMatch) and (numCurrencies > 0)) then
          -- Check if any alternative currency item matches the search string
          for currencyIdx = 1, numCurrencies do
            _, _, currencyLink = GetMerchantItemCostItem(itemIdx,
            currencyIdx);
            if ((currencyLink ~= nil) and
              (string.find(string.lower(string.match(currencyLink,
              "|h%[(.*)%]|h|r") or ""), searchText, 1, true) ~= nil)) then
              itemMatch = true;
              break;
            end
          end
        end
      end
    end

    -- Show/Hide textures for the button
    if ((isKnown) or (not itemMatch)) then
      merchantItemMasks[buttonIdx].isKnown:SetShown(isKnown);
      merchantItemMasks[buttonIdx].noMatchIcon:SetShown(not itemMatch);
      merchantItemMasks[buttonIdx].noMatchName:SetShown(not itemMatch);
      merchantItemMasks[buttonIdx]:Show();
    else
      merchantItemMasks[buttonIdx]:Hide();
    end
  end

  return;
end

--
-- Function used to update the Merchant frame
--
function merchantFrameUpdate()
  -- Local Variables
  local searchText = "";

  -- Check if searching is enabled
  if ((NigglesDBC.settings.UiMerchantSearch) and
    (merchantSearch ~= nil)) then
    -- Get the search text
    searchText = merchantSearch:GetText();
    if ((searchText == SEARCH) or (searchText == nil)) then
      searchText = "";
    else
      searchText = string.lower(searchText);
    end

    -- Show/Hide the Clear button
    merchantSearch.clearButton:SetShown(searchText ~= "");
  end

  -- Update the selected tab in the Merchant panel
  if (MerchantFrame.selectedTab == 1) then
    merchantTabUpdate(searchText);
  else
    buybackTabUpdate(searchText);
  end

  return;
end

--
-- Function called when the Merchant Search frame is shown
--
local function merchantSearchOnShow(self)
  -- Clear any old search text
  self:SetText("");

  return;
end

--
-- Function used to update the Merchant's frame, based on the search text
--
local function merchantSearchOnTextChanged(self, userInput)
  -- Update the search box's appearance
  SearchBoxTemplate_OnTextChanged(self);

  -- Update the Merchant's frame
  merchantFrameUpdate();

  return;
end

-------------------------------------------------------------------------------
--               G  L  O  B  A  L     F  U  N  C  T  I  O  N  S
-------------------------------------------------------------------------------

--
-- Function to initialise the Niggles Merchant features
--
function NigglesMerchantInitialise()
  -- Hook functions into the 'Merchant' frame
  MerchantPrevPageButton:HookScript("OnMouseUp", NigglesFirstLastPage);
  MerchantNextPageButton:HookScript("OnMouseUp", NigglesFirstLastPage);
  hooksecurefunc("MerchantFrame_Update", merchantFrameUpdate);

  return;
end

--
-- Function to enable/disable Merchant Search
--
function NigglesMerchantEnable()
  -- Local Variables
  local frame;
  local texture;
  local numButtons = math.max(MERCHANT_ITEMS_PER_PAGE, BUYBACK_ITEMS_PER_PAGE);

  -- Create the Search edit box, if required
  if ((NigglesDBC.settings.UiMerchantSearch) and
    (merchantSearch == nil)) then
    merchantSearch = CreateFrame("EDITBOX", nil, MerchantFrame,
    "SearchBoxTemplate");
    merchantSearch:SetSize(100, 20);
    merchantSearch:SetPoint("TOPLEFT", 64, -33);
    merchantSearch:SetMaxLetters(40);
    merchantSearch:SetScript("OnShow", merchantSearchOnShow);
    merchantSearch:SetScript("OnTextChanged", merchantSearchOnTextChanged);
  end

  -- Create the textures for the Merchant items, if required
  if ((NigglesDBC.settings.UiMerchantSearch) or
      (NigglesDBC.settings.UiMerchantItemKnown)) then
    for buttonIdx = 1, numButtons do
      if (merchantItemMasks[buttonIdx] == nil) then
        -- Create a frame to ensure the textures appear above the <erchant item
        frame = CreateFrame("Frame", nil, _G["MerchantItem"..buttonIdx]);
        frame:SetFrameLevel(129);
        frame:SetAllPoints();

        -- Create a texture to indicate recipes that are already known
        texture = frame:CreateTexture(nil, "OVERLAY", 1);
        texture:SetAllPoints(_G["MerchantItem"..buttonIdx.."ItemButton"])
        texture:SetTexture("Interface\\RAIDFRAME\\ReadyCheck-NotReady");
        frame.isKnown = texture;

        -- Create textures to indicate which items don't match a search string
        texture = frame:CreateTexture(nil, "OVERLAY", 2);
        texture:SetAllPoints(_G["MerchantItem"..buttonIdx.."ItemButton"]);
        texture:SetTexture(0, 0, 0, 0.8);
        frame.noMatchIcon = texture;
        texture = frame:CreateTexture(nil, "OVERLAY", 2);
        texture:SetPoint("TOPLEFT",
        _G["MerchantItem"..buttonIdx.."NameFrame"], 1, -1);
        texture:SetWidth(105);
        texture:SetHeight(48);
        texture:SetTexture(0, 0, 0, 0.8);
        frame.noMatchName = texture;

        -- Save the frame's handle
        merchantItemMasks[buttonIdx] = frame;
      end
    end
  end

  -- Check if Merchant Search is enabled
  if (merchantSearch ~= nil) then
    merchantSearch:SetShown(NigglesDBC.settings.UiMerchantSearch);
  end

  -- Update the Merchant's frame, if shown
  if (MerchantFrame:IsShown()) then
    merchantFrameUpdate();
  end

  return;
end
